<?php
/**
*
* @package - GeoMap
* @version $Id: geomap.php 25 2010-11-30 11:27:35Z Frank1604 $
* @copyright (c) Frank Wiesemann ( http://webmasteraena.net/ )
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* @package phpBB3
*/
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);

include($phpbb_root_path.'common.'.$phpEx);
include($phpbb_root_path . 'includes/functions_geomap.' . $phpEx);
include($phpbb_root_path . 'includes/functions_user.' . $phpEx);

$user->session_begin();
$auth->acl($user->data);
$user->setup('mods/geomap');

// we need an object..
$geomap = new geomap();

// get some data
$mode			= request_var('mode', '');
$post_land_id	= request_var('land', 0);
$post_plz		= request_var('plz', '0');
$post_distance	= request_var('distance', 10);

// define and initialize some vars
$geo_user_search_ary = $geo_search_events = array();
$postcode_not_found = true;
$pagination = $map = '';
$geo_search_count = 0;

// collect own postcode
$user->get_profile_fields($user->data['user_id']);
$own_plz = (isset($user->profile_fields['pf_plz']) ? $user->profile_fields['pf_plz'] : '');
$no_own_postcode = ($own_plz == '') ? true : false;

// if no postcode inserted, get it from user's profile
if (!$post_plz && $own_plz != '')
{
	$post_plz = $own_plz;
}

// and start
switch ($mode)
{
	case 'search':
		if ($post_land_id != 0 && $post_plz != 0)
		{
			// get some data
			$position				= $geomap->getPostcodePosition($post_plz, $post_land_id);
			$start					= request_var('start', 0);
			$geo_user_search_ary	= array();

			if (!empty($position))
			{
				$postcode_not_found = false;

				// get users in radius
				$geo_user_search_ary = $geomap->searchGeoUser($position['lenght'], $position['width'], $post_distance);
				$geo_search_count = count($geo_user_search_ary);

				// get events
				$geo_search_events = $geomap->getGeoEvents($position['lenght'], $position['width'], $post_distance);
				$geo_search_count += count($geo_search_events);

				$position['radius'] = $post_distance;
			}
			$geomap->createGeomap('search', 0, $geo_user_search_ary, $position);
		}
	break;

	case 'profile':
		$profile_user_id = request_var('u', 0);
		$geomap->createGeomap('profile', $profile_user_id);
	break;

	case 'events':
		$geomap->createGeomap('events');
	break;

	default:
		// check if map need to create new
		$current_time = time();
		$refresh_interval = $geomap->geoconfig['refresh_interval'] * 3600;

		if( filemtime($phpbb_root_path . 'images/geomap/usermap.png') < ($current_time - $refresh_interval))
		{
			// create geomap
			$geomap->createGeomap();
		}
	break;
}

if ($geo_search_count > $config['posts_per_page'])
{
	$pagination = generate_pagination(append_sid("{$phpbb_root_path}geomap.$phpEx?mode=search&amp;land=$post_land_id&amp;plz=$post_plz&amp;distance=$post_distance"), $geo_search_count, $config['posts_per_page'], $start);
	$geo_user_search_ary = array_slice($geo_user_search_ary, $start, $config['posts_per_page']);
}

// push default template values
$map_file = ($mode == 'search') ? $phpbb_root_path . 'images/geomap/map_search.html' : $phpbb_root_path . 'images/geomap/map.html';
$template->assign_vars(array(
	'MOD_NAME'				=> $user->lang['GEOMAP'],
	'U_FORM_ACTION'			=> append_sid($phpbb_root_path . 'geomap.' . $phpEx, 'mode=search'),
	'RADIUS'				=> $post_distance,
	'S_POSTCODE_NOT_FOUND'	=> $postcode_not_found,
	'POSTCODE'				=> ($post_land_id == 0) ? sprintf("%05d", $post_plz) : $post_plz,
	'GEO_MAP_VALUES'		=> str_replace('{U_SESSION_ID}', $user->session_id, ($mode == 'profile' ? $map : file_get_contents($map_file))),
	'GEO_USER_COUNT'		=> sprintf($user->lang['GEO_USER_COUNT'], $geomap->getGeoCount()),
	'S_NO_OWN_POSTCODE'		=> $no_own_postcode,
	'S_SEARCH'				=> ($mode == 'search' ? true : false),
	'S_PROFILE'				=> ($mode == 'profile' ? true : false),
	'PAGINATION'			=> $pagination,
	'GEO_SEARCH_COUNT'		=> $geo_search_count,
	'S_ACTIVE'				=> $geomap->geoconfig['map_status'],
	'INACTIVE_TEXT'			=> $geomap->geoconfig['inactive_text'],
	'GEO_INFO'				=> sprintf($user->lang['GEO_INFO'], $geomap->geoconfig['refresh_interval']),
	'U_SESSION_ID'			=> $user->session_id,
	'S_IN_GEOMAP'			=> true,
));

// block vars for geosearch result
foreach($geo_user_search_ary as $geo_search_user)
{
	$template->assign_block_vars('search_result_row', array(
		'USER_ID' 		=> $geo_search_user['user_id'],
		'USER_NAME'		=> $geo_search_user['username'],
		'PLZ'			=> $geo_search_user['pf_plz'],
		'TOWN'			=> $geo_search_user['user_from'],
		'LAND_ID'		=> $geo_search_user['pf_land'],
		'DISTANCE'		=> $geo_search_user['distance'],
		'USER_NAME'		=> get_username_string('full', $geo_search_user['user_id'], $geo_search_user['username'], $geo_search_user['user_colour'], $geo_search_user['username_clean']),
	));
}

// block vars for events
foreach($geo_search_events as $geo_search_event)
{
	$template->assign_block_vars('events_row', array(
		'NAME' 			=> $geo_search_event['event_name'],
		'DESC'			=> $geo_search_event['event_desc'],
		'URL'			=> $geo_search_event['event_url'],
		'ORT'			=> $geo_search_event['event_ort'],
		'DISTANCE'		=> $geo_search_event['distance'],
	));
}

// block vars for land selection
foreach($geomap->getLandOptions() as $land_id => $land_value)
{
	$template->assign_block_vars('land_option', array(
		'LAND_ID'		=> $land_id,
		'LAND_VALUE'	=> $land_value,
		'LAND_SELECTED'	=> ($post_land_id == $land_id ? true : false),
	));
}

// start output
page_header($user->lang['GEOMAP']);

$template->set_filenames(array(
	'body' => 'geomap_body.html'
));

page_footer();
?>