<?php
/**
*
* @package - GeoMap
* @version $Id: geomap.php 25 2010-11-30 11:27:35Z Frank1604 $
* @copyright (c) Frank Wiesemann ( http://webmasterarena.net/ )
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/
if (!defined('IN_PHPBB'))
{
	exit;
}
/**
* @package phpBB3
*/

class geomap
{
	var $geoconfig = array();
	var $geogroups = array();
	var $geocount = 0;

	// constructor
	function geomap()
	{
		$this->getConfig();
	}

	// load geomap config values
	function getConfig()
	{
		global $db;

		// read config table
		$sql = 'SELECT * FROM ' . GEOMAP_CONFIG_TABLE;
		$result = $db->sql_query($sql);
		$row = $db->sql_fetchrow($result);
		$db->sql_freeresult($result);

		$row['user_color'] = strtoupper($row['user_color']);
		$row['admin_color'] = strtoupper($row['admin_color']);
		$row['mod_color'] = strtoupper($row['mod_color']);
		$row['map_status'] = ($row['active'] == 1 ? true : false);

		$this->geoconfig = $row;
	}

	// check if the given string is a 4 or 5 digits postcode
	function is_plz_string ($plz_string)
	{
		return (preg_match("/^\d{4,5}$/", $plz_string)) ? true : false;
	}

	// counts the user on map
	function getGeoCount()
	{
		global $db;

		$sql = 'SELECT COUNT(u.user_id) AS user_count
			FROM ' . USERS_TABLE . ' u
			INNER JOIN ' . PROFILE_FIELDS_DATA_TABLE . ' pd ON u.user_id = pd.user_id
			LEFT JOIN ' . GROUPS_TABLE . ' g ON u.group_id = g.group_id
			LEFT JOIN ' . GEOMAP_POSTCODE_TABLE . ' gp ON ( pd.pf_plz = gp.plz
			AND pd.pf_land = gp.land )
			WHERE pd.pf_plz IS NOT NULL
			AND gp.breite IS NOT NULL 
			AND g.geo_hide = 0
			ORDER BY gp.breite, gp.laenge';

		$result = $db->sql_query($sql);
		$geo_user_count = (int) $db->sql_fetchfield('user_count');
		$db->sql_freeresult($result);

		$this->geocount = $geo_user_count;
		return $geo_user_count;
	}

	// collect the boardgroups 4 display on geomap
	function getGeoGroups($all = false)
	{
		global $db, $user;
		
		$sql_and = ($all) ? '' : 'AND g.geo_hide = 0';
		$sql = 'SELECT g.group_id, g.group_name, g.group_colour, g.group_type, g.geo_hide
			FROM ' . GROUPS_TABLE . ' g
			WHERE (g.group_type <> ' . GROUP_HIDDEN . ') ' .
				$sql_and . '
			ORDER BY g.group_name ASC';
		$result = $db->sql_query($sql);

		$geogroups_ary = array();
		while ($row = $db->sql_fetchrow($result))
		{
			$geogroups_ary[] = array(
				'group_id'		=> $row['group_id'],
				'group_colour'	=> ($row['group_colour']) ? $row['group_colour'] : $this->geoconfig['user_color'],
				'group_name'	=> ($row['group_type'] == GROUP_SPECIAL) ? $user->lang['G_' . $row['group_name']] : $row['group_name'],
				'pixel_size'	=> ($row['group_name'] == 'ADMINISTRATORS' ? $this->geoconfig['admin_pixel'] : ($row['group_name'] == 'GLOBAL_MODERATORS' ? $this->geoconfig['mod_pixel'] : $this->geoconfig['user_pixel'])),
				'geo_hide'		=> $row['geo_hide'],
			);
		}
		$db->sql_freeresult($result);
		$this->geogroups = $geogroups_ary;

		return $geogroups_ary;
	}

	// get postcode position
	function getPostcodePosition($plz, $land_id)
	{
		global $db;
		$position = array();

		$sql = 'SELECT laenge, breite FROM ' . GEOMAP_POSTCODE_TABLE . " 
			WHERE plz = '$plz '
			AND land = $land_id";

		$result = $db->sql_query($sql);
		
		if ($result)
		{
			$row = $db->sql_fetchrow($result);
			$db->sql_freeresult($result);

			if($row['laenge'] <> '' && $row['breite'] <> '')
			{
				$position = array(
					'lenght'	=> $row['laenge'],
					'width'	=> $row['breite'],
				);
			}
		}

		return $position;
	}

	// get land option ary
	function getLandOptions()
	{
		global $db, $user;
		$land_option_ary = array();

		$sql = "SELECT t2.option_id, t2.lang_value
			FROM " . PROFILE_FIELDS_TABLE . " t1
			INNER JOIN " . PROFILE_FIELDS_LANG_TABLE . " t2
			ON t1.field_id = t2.field_id
			WHERE t2.lang_id = " . $user->get_iso_lang_id() . "
			AND t1.field_name = 'land'
			ORDER BY t2.option_id";
		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$land_option_ary[$row['option_id']+1] = $row['lang_value'];
		}
		$db->sql_freeresult($result);

		//delete the last value, "not selected"
		array_pop($land_option_ary);

		return $land_option_ary;
	}

	// check if profile field exists
	// $fields_to_check can be a single value or an array
	function profile_field_exists($fields_to_check = false)
	{
		global $db;

		// quit with false if there is no value
		if (!$fields_to_check)
		{
			return false;
		}

		if (is_array($fields_to_check))
		// is an array given..
		{
			// build fieldname sring
			foreach ($fields_to_check as $field)
			{
				$field_name .= "field_name = '$field' AND ";
			}
			// cut the last 'AND'
			$field_name = substr($field_name, 0, -5);
		}
		else
		// .. or only one field?
		{
			// build fieldname sring
			$field_name = "field_name = '$fields_to_check'";
		}

		// build sql sring
		$sql = 'SELECT COUNT(field_id) as field_count
			FROM ' . PROFILE_FIELDS_TABLE . ' 
			WHERE ' .$field_name;

		// and count..
		$result = $db->sql_query($sql);
		$field_count = (int) $db->sql_fetchfield('field_count');
		$db->sql_freeresult($result);

		if ($field_count == 0)
		{
			//field(s) doesn't exists
			return false;
		}
		else
		{
			//field(s) exists
			return true;
		}
	}

	// convert a hexadecimal code to an rgb array
	function hex2rgb($color)
	{
		$color = str_replace('#','',$color);
		$s = strlen($color) / 3;
		$rgb[]=hexdec(str_repeat(substr($color,0,$s),2/$s));
		$rgb[]=hexdec(str_repeat(substr($color,$s,$s),2/$s));
		$rgb[]=hexdec(str_repeat(substr($color,2*$s,$s),2/$s));
		return $rgb;
	}

	/**
	* stroke a filled elipse with gradient
	* works like imagefilledelipse but with two color parameters: $ic is the inner color 
	* of the gradient ellipse and $oc is the outer color.
	*
	* imagegradientellipse by Felix Bruns
	*/
	function imagegradientellipse($image, $cx, $cy, $w, $h, $ic, $oc)
	{
		$w = abs($w);
		$h = abs($h);
		$oc = array(0xFF & ($oc >> 0x10), 0xFF & ($oc >> 0x8), 0xFF & $oc);
		$ic = array(0xFF & ($ic >> 0x10), 0xFF & ($ic >> 0x8), 0xFF & $ic);
		$c0 = ($oc[0] - $ic[0]) / $w;
		$c1 = ($oc[1] - $ic[1]) / $w;
		$c2 = ($oc[2] - $ic[2]) / $w;
		$i = 0;
		$j = 0;
		$is = ($w < $h) ? ($w / $h) : 1;
		$js = ($h < $w) ? ($h / $w) : 1;
		while(1)
		{
			$r = $oc[0] - floor($i * $c0);
			$g = $oc[1] - floor($i * $c1);
			$b = $oc[2] - floor($i * $c2);
			$c = imagecolorallocate($image, $r, $g, $b);
			imagefilledellipse($image, $cx, $cy, $w-$i, $h-$j, $c);
			if($i < $w)
			{
				$i += $is;
			}
			if($j < $h)
			{
				$j += $js;
			}
			if($i >= $w && $j >= $h)
			{
				break;
			}
		}
	}

	// Convert $img to truecolor
	// taken from php.net, written by jeff
	function imgtotruecolor(&$img)
	{
		$w = imagesx($img);
		$h = imagesy($img);
		if (!imageistruecolor($img))
		{
			$original_transparency = imagecolortransparent($img);

			//we have a transparent color
			if ($original_transparency >= 0)
			{
				//get the actual transparent color
				$rgb = imagecolorsforindex($img, $original_transparency);
				$original_transparency = ($rgb['red'] << 16) | ($rgb['green'] << 8) | $rgb['blue'];

				//change the transparent color to black, since transparent goes to black anyways (no way to remove transparency in GIF)
				imagecolortransparent($img, imagecolorallocate($img, 0, 0, 0));
			}

			//create truecolor image and transfer
			$truecolor = imagecreatetruecolor($w, $h);
			imagealphablending($img, false);
			imagesavealpha($img, true);
			imagecopy($truecolor, $img, 0, 0, 0, 0, $w, $h);
			imagedestroy($img);
			$img = $truecolor;

			//remake transparency (if there was transparency)
			if ($original_transparency >= 0)
			{
				imagealphablending($img, false);
				imagesavealpha($img, true);
				for ($x = 0; $x < $w; $x++)
				{
					for ($y = 0; $y < $h; $y++)
					{
						if (imagecolorat($img, $x, $y) == $original_transparency)
						{
							imagesetpixel($img, $x, $y, 127 << 24);
						}
					}
				}
			}
		}
	}

	// get the distance between 2 points on map in km
	function get_distance($pos1_len, $pos1_width, $pos2_len, $pos2_width, $earth_radius = 6370)
	{
		$distance = aCos(Sin(deg2rad($pos1_width)) * Sin(deg2rad($pos2_width)) + Cos(deg2rad($pos1_width)) * Cos(deg2rad($pos2_width)) * Cos(deg2rad($pos1_len - $pos2_len))) * $earth_radius;
		return $distance;
	}

	// sort an array by field
	function sort_ary_by_field($original, $field, $descending = false)
	{
		$sort_ary = array();
		if (is_array($original))
		{
			foreach ($original as $key => $value)
			{
				$sort_ary[$key] = $value[$field];
			}

			if ($descending)
			{
				arsort($sort_ary);
			}
			else
			{
				asort($sort_ary);
			}

			$result_ary = array();
			foreach ($sort_ary as $key => $value)
			{
				$result_ary[$key] = $original[$key];
			}

			return $result_ary;
		}
	}

	// gets all users with postcode entries except own user_id, if specified
	// calculate distance if position is specified
	function searchGeoUser($position_lenght = false, $position_width = false, $radius = 10, $own_id = 0, $profile_id = 0)
	{
		global $db, $user;
		$geo_user_ary = $tmpGeoUsers = array();

		if ($profile_id != 0)
		{
			$tmpGeoUsers = $this->getGeoUsers('profile', $profile_id);
		}
		else
		{
			if ($radius == 0)
			{
				$tmpGeoUsers = $this->getGeoUsers('all', 0, $position_lenght, $position_width);
			}
			else
			{
				$tmpGeoUsers = $this->getGeoUsers('all');
			}
		}

		foreach($tmpGeoUsers as $geoUser)
		{
			if ($position_lenght != false && $position_width != false)
			{
				// position is specified, calculate distance
				$distance = round($this->get_distance($position_lenght, $position_width, $geoUser['laenge'], $geoUser['breite']),1);

				if ($distance <= $radius)
				{
					$geoUser['distance'] = $distance;
					$geo_user_ary[] =  $geoUser;
				}
			}
			else
			{
				$geo_user_ary[] =  $geoUser;
			}
		}

		$return_value = ($position_lenght != false && $position_width != false ? $this->sort_ary_by_field($geo_user_ary, 'distance') : $geo_user_ary);
		return $return_value;
	}

	// get events
	function getGeoEvents($position_lenght = false, $position_width = false, $radius = 0, $event_id = 0, $all = false)
	{
		global $db;
		$geoEvents_ary = array();

		if ($this->geoconfig['show_events'])
		{
			$time = time();

			$sql = 'SELECT * FROM ' . GEOMAP_EVENTS_TABLE;
			$sql .=  ($event_id > 0) ? ' WHERE event_id = ' . $event_id : " WHERE event_start < $time AND event_end > $time AND event_aktiv = 1";
			if ($position_lenght && $position_width && $radius == 0) $sql .= " AND event_laenge = $position_lenght AND event_breite = $position_width";

			if ($all)
			{
				$sql = 'SELECT * FROM ' . GEOMAP_EVENTS_TABLE;
			}
			$sql .= ' ORDER BY event_end DESC';

			$result = $db->sql_query($sql);

			while ($row = $db->sql_fetchrow($result))
			{
				if($position_lenght && $position_width)
				{
					$row['distance'] = round($this->get_distance($position_lenght, $position_width, $row['event_laenge'], $row['event_breite']),1);
					if ($radius == 0 || $row['distance'] <= $radius)
					{
						$geoEvents_ary[] = $row;
					}
				}
				elseif ($event_id > 0)
				{
					$geoEvents_ary = $row;
				}
				else
				{
					$geoEvents_ary[] = $row;
				}
			}
			$db->sql_freeresult($result);
		}

		return $geoEvents_ary;
	}

	// collect geomapuser from board
	function getGeoUsers($mode = 'user', $user_id = 0, $position_lenght = 0, $position_width = 0)
	{
		global $db;
		$geoUsers_ary = array();

		switch ($mode)
		{
			case 'mod':
				$sql_and = "AND g.group_name = 'GLOBAL_MODERATORS'";
				break;
			case 'admin':
				$sql_and = "AND g.group_name = 'ADMINISTRATORS'";
				break;
			case 'groups':
				$sql_and = "AND g.group_name <> 'ADMINISTRATORS' AND g.group_name <> 'GLOBAL_MODERATORS' AND g.group_name <> 'REGISTERED' AND g.group_name <> 'BOTS'";
				break;
			case 'profile':
				$sql_and = "AND u.user_id = " . $user_id;
				break;
			case 'all':
				$sql_and = '';
				break;
			default:
			$sql_and = "AND g.group_name = 'REGISTERED'";
		}

		if($position_lenght != 0 && $position_width != 0)
		{
			$sql_and = " AND gp.laenge = $position_lenght	AND gp.breite = $position_width";
		}

		$sql = 'SELECT u.user_id, u.username, u.user_from, u.user_colour, u.username_clean, u.group_id, g.group_name, g.group_colour, pd.pf_plz, pd.pf_land, gp.breite, gp.laenge, gp.ort
			FROM ' . USERS_TABLE . ' u
			INNER JOIN ' . PROFILE_FIELDS_DATA_TABLE . ' pd ON u.user_id = pd.user_id
			LEFT JOIN ' . GROUPS_TABLE . ' g ON u.group_id = g.group_id
			LEFT JOIN ' . GEOMAP_POSTCODE_TABLE . ' gp ON ( pd.pf_plz = gp.plz
			AND pd.pf_land = gp.land )
			WHERE pd.pf_plz IS NOT NULL
			AND gp.breite IS NOT NULL ' .
			$sql_and . '
			AND g.geo_hide = 0
			ORDER BY gp.breite, gp.laenge';

		$result = $db->sql_query($sql);

		while ($row = $db->sql_fetchrow($result))
		{
			$row['pixel_size'] = ($row['group_name'] == 'ADMINISTRATORS' ? $this->geoconfig['admin_pixel'] : ($row['group_name'] == 'GLOBAL_MODERATORS' ? $this->geoconfig['mod_pixel'] : $this->geoconfig['user_pixel']));
			if($row['group_colour'] == '')
			{
				$row['group_colour'] = $this->geoconfig['user_color'];
			}
			if($row['user_from'] == '')
			{
				$row['user_from'] = $row['ort'];
			}
			$geoUsers_ary[] = $row;
		}
		$db->sql_freeresult($result);

		return $geoUsers_ary;
	}

	// creates the map, draw the points and caption
	function createGeomap($mode = '', $user_id = 0, $usr_search_ary = array(), $position = array('lenght' => 0, 'width' => 0, 'radius'  => 0))
	{
		global $db, $user, $phpbb_root_path, $phpEx, $map;
		$geoUsers = $geo_map_values = array();

		switch($mode)
		{
			case 'profile':
				$geoUsers = $this->getGeoUsers('profile', $user_id);
			break;

			case 'events':

			break;

			case 'search':
/* 				if ($position['radius'] >= 10)
				{ */
					$geoUsers = $usr_search_ary;
					break;
/* 				} */

			default:
				// select users
				$geoUsers = array_merge($geoUsers, $this->getGeoUsers('user'));
				// select other groups then registered
				$geoUsers = array_merge($geoUsers, $this->getGeoUsers('groups'));
				// select global moderators
				$geoUsers = array_merge($geoUsers, $this->getGeoUsers('mod'));
				// select administrators
				$geoUsers = array_merge($geoUsers, $this->getGeoUsers('admin'));
			break;
		}

		// get image and convert it
		$image = imagecreatefrompng($phpbb_root_path . 'images/geomap/karte.png');
		$this->imgtotruecolor($image);

		// define some colors
		$white	= imagecolorallocate($image, 255, 255, 255);
		$grey	= imagecolorallocate($image, 50, 50, 50);

		// get size of image
		$scale_x = imagesx($image);
		$scale_y = imagesy($image);

		// create geomap values
		$tmp_last_lenght = $tmp_last_width = $map_key = 0;
		foreach ($geoUsers as $geo_user)
		{
			// define color and pixelsize for this user
			$color = $this->hex2rgb($geo_user['group_colour']);
			$color = imagecolorallocate($image, $color[0], $color[1], $color[2]);
			$pixel_size = $geo_user['pixel_size'];

			// build value array for usermap
			if ($tmp_last_lenght == $geo_user['laenge'] && $tmp_last_width == $geo_user['breite'])
			{
				$geo_map_values[$map_key]['user_count'] = $geo_map_values[$map_key]['user_count'] + 1;
				$geo_map_values[$map_key]['user_desc'] .= ', ' . $geo_user['user_id'] . ' (' . $geo_user['user_from'] . ')';
				$geo_map_values[$map_key]['pixel_size'] = (($pixel_size / 2) > $geo_map_values[$map_key]['pixel_size'] ? $pixel_size / 2 : $geo_map_values[$map_key]['pixel_size']);
			}
			else
			{
				$map_key++;
				$geo_map_values[$map_key]['user_desc'] = $geo_user['username'] . ' (' . $geo_user['user_from'] . ')';
				$geo_map_values[$map_key]['user_count'] = 1;
				$geo_map_values[$map_key]['plz'] = $geo_user['pf_plz'];
				$geo_map_values[$map_key]['land'] = $geo_user['pf_land'];
				$geo_map_values[$map_key]['latitude'] = round(($geo_user['breite'] - $this->geoconfig['max_latitude']) * $scale_y / ($this->geoconfig['min_latitude'] - $this->geoconfig['max_latitude']));
				$geo_map_values[$map_key]['longitude'] = round(($geo_user['laenge'] - $this->geoconfig['min_longitude']) * $scale_x / ($this->geoconfig['max_longitude'] - $this->geoconfig['min_longitude']));
				$geo_map_values[$map_key]['pixel_size'] = $pixel_size;
				$geo_map_values[$map_key]['color'] = $color;
				$tmp_last_lenght = $geo_user['laenge'];
				$tmp_last_width = $geo_user['breite'];

				$geo_map_values[$map_key]['laenge'] = $geo_user['laenge'];
				$geo_map_values[$map_key]['breite'] = $geo_user['breite'];
			}
		}

		// and now.. the events
		if ($mode == 'search' && isset($position['radius']) && $position['radius'] >= 10)
		{
			$geoEvents = $this->getGeoEvents($position['lenght'], $position['width'], $position['radius']);
		}
 		else
		{
			$geoEvents = $this->getGeoEvents();
		}

		foreach ($geoEvents as $geo_event)
		{
			$event_color = $this->hex2rgb($this->geoconfig['event_color']);
			$event_color = imagecolorallocate($image, $event_color[0], $event_color[1], $event_color[2]);

			$geo_map_values[] = array(
				'user_desc' 	=> $geo_event['event_name'] . ' (' . $geo_event['event_ort'] . ') - ' . $geo_event['event_desc'] . "\n - " . $geo_event['event_url'],
				'plz' 			=> $geo_event['event_plz'],
				'land' 			=> $geo_event['event_land'],
				'latitude' 		=> round(($geo_event['event_breite'] - $this->geoconfig['max_latitude']) * $scale_y / ($this->geoconfig['min_latitude'] - $this->geoconfig['max_latitude'])),
				'longitude' 	=> round(($geo_event['event_laenge'] - $this->geoconfig['min_longitude']) * $scale_x / ($this->geoconfig['max_longitude'] - $this->geoconfig['min_longitude'])),
				'pixel_size' 	=> $this->geoconfig['event_pixel'],
				'color' 		=> $event_color,
				'breite'		=> $geo_event['event_breite'],
				'laenge'		=> $geo_event['event_laenge'],
			);
		}

		// caption position from left.
		$caption_left = 7;
		// caption position from top.
		$caption_top = 20;

		// draw caption on map
		foreach($this->getGeoGroups() as $geo_group)
		{
			$color = $this->hex2rgb($geo_group['group_colour']);
			$color = imagecolorallocate($image, $color[0], $color[1], $color[2]);
			$pixel_size = $geo_group['pixel_size'];

			$this->imagegradientellipse($image, $caption_left , $scale_y - ($scale_y - $caption_top), $pixel_size, $pixel_size, $white, $color);
			imagestring($image, 2, $caption_left + 10, $scale_y - ($scale_y - ($caption_top - $pixel_size / 2)), utf8_decode($geo_group['group_name']), $grey);
			$caption_top += 20;
		}

		// events on caption
		if ($this->geoconfig['show_events'])
		{
			$color = $this->hex2rgb($this->geoconfig['event_color']);
			$color = imagecolorallocate($image, $color[0], $color[1], $color[2]);
			$pixel_size = $this->geoconfig['event_pixel'];

			$this->imagegradientellipse($image, $caption_left , $scale_y - ($scale_y - $caption_top), $pixel_size, $pixel_size, $white, $color);
			imagestring($image, 2, $caption_left + 10, $scale_y - ($scale_y - ($caption_top - $pixel_size / 2)), utf8_decode($user->lang['GEO_EVENTS']), $grey);
		}

		// draw users on map
		if (is_array($geo_map_values))
		{
			foreach ($geo_map_values as $value_ary)
			{
				// create "<map></map>" values
				$map .= '<area shape="circle" coords="' . $value_ary['longitude'] . ',' . $value_ary['latitude'] . ', ' . ($value_ary['pixel_size'] / 2) . '" href="geomap.' . $phpEx . '?mode=search&amp;land=' . $value_ary['land'] . '&amp;plz=' . $value_ary['plz'] . '&amp;distance=0&amp;sid={U_SESSION_ID}" alt="' . $value_ary['user_desc'] . '" onmouseover="show_geo_popup(\'' . $value_ary['plz'] . '\',\'' . $value_ary['land'] . '\')" onmouseout="close_geo_popup()" />' . "\n";

				// draw the point
				$this->imagegradientellipse($image, $value_ary['longitude'], $value_ary['latitude'], $value_ary['pixel_size'], $value_ary['pixel_size'], $white, $value_ary['color']);
				imageellipse($image, $value_ary['longitude'], $value_ary['latitude'], $value_ary['pixel_size'], $value_ary['pixel_size'], $grey);
			}

			// write <area />" values to file
			if ($mode != 'profile')
			{
				$map_file = ($mode == 'search') ? "{$phpbb_root_path}images/geomap/map_search.html" : "{$phpbb_root_path}images/geomap/map.html";
				$handle = fopen ($map_file, 'w');
				fwrite($handle, $map);
				fclose($handle);
			}
		}

		// draw search radius
		if ($mode == 'search' && isset($position['radius']) && $position['radius'] >= 10)
		{
			$sr_center_x = round(($position['lenght'] - $this->geoconfig['min_longitude']) * $scale_x / ($this->geoconfig['max_longitude'] - $this->geoconfig['min_longitude']));
			$sr_center_y = round(($position['width'] - $this->geoconfig['max_latitude']) * $scale_y / ($this->geoconfig['min_latitude'] - $this->geoconfig['max_latitude']));
			$sr_center_z = round((($position['width'] + 2.05) - $this->geoconfig['max_latitude']) * $scale_y / ($this->geoconfig['min_latitude'] - $this->geoconfig['max_latitude']));
			$radius = (($sr_center_y - $sr_center_z) / 213) * $position['radius'];

			imageellipse($image, $sr_center_x , $sr_center_y, $radius * 2, $radius * 2, $grey);
		}

		// convert image to palette
		imagetruecolortopalette($image, false, 256);

		// detect transparency color from the upper left corner and set it
		$transparency_color = imagecolorat ($image, 1, 1);
		imagecolortransparent($image, $transparency_color);

		// save image
		if ($mode == 'profile')
		{
			imagepng($image, $phpbb_root_path . 'images/geomap/usermap_profile.png');
		}
		elseif ($mode == 'search')
		{
			imagepng($image, $phpbb_root_path . 'images/geomap/usermap_search.png');
		}
		else
		{
			imagepng($image, $phpbb_root_path . 'images/geomap/usermap.png');
		}
		imagedestroy($image);
	}
}
?>